import hnswlib
import numpy as np
from google.cloud import bigquery
from google.cloud.bigquery.table import RowIterator

from importlib_resources import files
from importlib_resources import as_file

project = "podcast-recs"
client = bigquery.Client()


def readTable(dataset, table) -> RowIterator:
    d_ref = bigquery.DatasetReference(project, dataset)
    t_ref = d_ref.table(table)
    return client.list_rows(t_ref)


def read_data(dataset, table):
    rit = readTable(dataset, table)
    vectors = [r.get("vector") for r in rit]
    return vectors


def make_index(vectors):
    dim = len(vectors[0])
    elements = len(vectors)
    ids = np.arange(elements)

    p = hnswlib.Index(space="cosine", dim=dim)
    p.init_index(max_elements=elements, ef_construction=200, M=16)
    p.add_items(vectors, ids)
    p.set_ef(50)

    return p


def index_from_table(dataset, table):
    vectors = read_data(dataset, table)
    return make_index(vectors)


def get_distances(p, vectors, k=1):
    labels, distances = p.knn_query(vectors, k=k)
    return labels, distances


"""save to file fn"""


def save(fn, p):
    p.save_index(fn)


"""loads from resource file in resources directory"""


def load(fn="index.bl", dim=64):
    p = hnswlib.Index(space="cosine", dim=dim)
    source = files("knn").joinpath(fn)
    with as_file(source) as sfn:
        fn = f"{sfn.parent}/{sfn.name}"
        print(fn)
        p.load_index(fn)
    return p


def test():
    src = files("knn").joinpath("a.txt")
    with as_file(src) as sfn:
        fn = f"{sfn.parent}/{sfn.name}"
        data = ftxt(fn)
        print(data)
        return data


def ftxt(fn):
    with open(fn, "r") as f:
        return f.read()


if __name__ == "__main__":

    dataset = "test_datasets"
    table = "tiny_vectors"
    test()
    # table = "english_show_shows_20211009"
    # vectors = read_data(dataset, table)
    # p = load()
    # ls, ds = get_distances(p.vectors)
    # print(ls, ds)



import logging
import knn.distances as ds

import apache_beam as beam
from apache_beam.options.pipeline_options import PipelineOptions

# to run:
# python -m knn_main --setup_file <path to setup.py>

class KNNDoFn(beam.DoFn):
    def setup(self):
        self.dim = 64
        self.index = ds.load("index.bl", self.dim)

    """Parse each line of input text into words."""

    def process(self, element):
        labels, scores = ds.get_distances(self.index, element.get("vector"))
        return [abs(score) for score in scores]


def run(argv=None):
    """Main entry point; defines and runs the wordcount pipeline."""

    # The pipeline will be run on exiting the with block.
    beam_options = PipelineOptions(
        # runner="DataflowRunner",
        runner="DirectRunner",
        project="podcast-recs",
        region="europe-west1",
        job_name="rkumar-test",
        temp_location="gs://recs_test/tmp/",
    )
    with beam.Pipeline(options=beam_options) as p:

        # Read the BQ table into a PCollection.

        # Read the BQ table into a PCollection.
        table_spec = "podcast-recs:test_datasets.tiny_vectors"
        lines = p | "Read" >> beam.io.ReadFromBigQuery(table=table_spec)
        counts = (
            lines
            | "Count" >> beam.ParDo(KNNDoFn())
            | "Mean" >> beam.combiners.Mean.Globally()
            | beam.Map(print)
        )


# To run:
# python -m knn --requirements_file requirements.txt
# > cat requirements.txt
# > google-cloud-bigquery
# >hnswlib
if __name__ == "__main__":
    logging.getLogger().setLevel(logging.INFO)
    run()



from setuptools import setup

setup(
    name='knn_rk47',
    version='10',
    install_requires=[
        'hnswlib',
    ],
    packages=['knn'],
    package_dir={
        "": "src"
        },
    package_data={
        "knn": ["index.bl"]
    },
    include_package_data=True
)

